#!/usr/bin/env python3

import pathlib
import pandas as pd
import datetime
import pickle
import os
import sys

input_file = sys.argv[1]    #'./data/politicians/Ratsmitglieder_1848_DE_corr.xlsx'
input_file_addInfo = sys.argv[2]    #'./data/politicians/MPs_additionalInfo.csv'
output_file_csv = sys.argv[3]     #'./data/politicians/MPs_after1890.csv'
output_folder_pickle = sys.argv[4]     #'./data/politicians/yearly_pickles/'
output_folder_csv = sys.argv[5]     #'./data/politicians/yearly_csvs/'


class MPs_Extractor(object):

    def __init__(self, years, input_file, input_file_addInfo, output_file_csv, output_folder_pickle, output_folder_csv):
        self.input_file = input_file
        self.input_file_addInfo = input_file_addInfo
        self.output_file_csv = output_file_csv
        self.output_folder_pickle = output_folder_pickle
        self.output_folder_csv = output_folder_csv
        self.range_years = range(years[0], years[1] + 1)

    # function to refine dataframe for name disambiguation
    # input:
    # - df_year: dataframe for a year
    # output:
    # - df_year: updated dataframe with shortName, completeName and nameType
    #        - complete name: lastname firstname (canton cantonabbr)
    #        - if lastname appears multiple times
    #                nameType = 'canton'
    #                shortname = according to following three types
    #        - if lastname is a composite name such as 'von Arx' or 'de Stoppani'
    #                nameType = 'comp'
    #                shortName = Arx or Stoppani
    #        - if lastname is a double name such as 'Meier-Müller'
    #                nameType = 'double'
    #                for each double name, four entries are made:
    #                        - shortName = 'Meier'
    #                        - shortName = 'Müller'
    #                        - shortName = 'Meier-Müller'
    #                        - shortName = 'MeierMüller'
    #        - if lastname is none of the above
    #               nameType = 'simple'
    #                shortName = lastname
    def refine_yearly_dataframe(self, df_year):
        str_simple = 'simple'
        str_double = 'double'
        str_comp = 'comp'
        str_canton = 'canton'

    	# function to split lastname and save meaningful part(s) to list
        def split_lastname(df_year, lastname, uniqueID, str_completeName, bln_unique = True):

            def update_dataframe(df_year, str_shortName, str_completeName, str_nameType, bln_unique):

                # set short name
                df_year.loc[(df_year['uniqueIndex'] == uniqueID), 'shortName'] = str_shortName

                # set complete name
                df_year.loc[(df_year['uniqueIndex'] == uniqueID), 'completeName'] = str_completeName

                # set name type
                if bln_unique:
                    df_year.loc[(df_year['uniqueIndex'] == uniqueID), 'nameType'] = str_nameType
                else:
                    df_year.loc[(df_year['uniqueIndex'] == uniqueID), 'nameType'] = str_canton

                # generate three more entries for double names
                if str_nameType == 'double':
                    # duplicate this entry three times
                    df_tripled = df_year[df_year['uniqueIndex'] == uniqueID]
                    df_tripled = pd.concat([df_tripled]*3, ignore_index = True)

                    # set short name without -
                    i = 0
                    df_tripled.loc[i, 'shortName'] = ''.join(lastname.split('-'))
                    # and for each separate name
                    for item in lastname_split2:
                        i += 1
                        df_tripled.loc[i, 'shortName'] = item

                    # concatenate with yearly dataframe
                    df_year = pd.concat([df_year, df_tripled], ignore_index = True)

                return df_year

    	    # if last name is a composite name, e.g. 'von Arx' and 'de Stoppani'
            lastname_split = lastname.split()
            if len(lastname_split) > 1:
                for item in lastname_split:
                    if item not in ['von', 'de', 'Ab', 'van']:
            			# update dataframe
                        df_year = update_dataframe(df_year, item, str_completeName, str_comp, bln_unique)

            else:
        		# if last name is a double name, e.g. 'Meier-Müller'
                lastname_split2 = lastname.replace('-', ' ').split()
                if len(lastname_split2) > 1:
        			# update dataframe
                    df_year = update_dataframe(df_year, lastname, str_completeName, str_double, bln_unique)

                else:
        			# update dataframe
                    df_year = update_dataframe(df_year, lastname, str_completeName, str_simple, bln_unique)

            return df_year

        def _get_complete_name(df_year, uniqueID):
            _str_lastName = df_year['LastName'].loc[df_year['uniqueIndex']==uniqueID].iloc[0]
            _str_firstName = df_year['FirstName'].loc[df_year['uniqueIndex']==uniqueID].iloc[0]
            _str_cantonName = df_year['CantonName'].loc[df_year['uniqueIndex']==uniqueID].iloc[0]
            _str_cantonAbbr = df_year['CantonAbbreviation'].loc[df_year['uniqueIndex']==uniqueID].iloc[0]
            _str_canton = '(' + ' '.join((_str_cantonName, _str_cantonAbbr)) + ')'
            str_completeName = ' '.join((_str_lastName, _str_firstName, _str_canton))

            return str_completeName

    	# for every last name
        for lastname in df_year['LastName'].drop_duplicates():
	    #print('name', lastname, type(lastname))

    	    # extract all entries with that last name
            df_temp = df_year.loc[df_year['LastName']==lastname]
    	    #print(df_temp)

    	    # if only one person with that last name
            if df_temp.drop_duplicates(['uniqueIndex']).shape[0] == 1:
        		# extract unique index
                uniqueID = df_temp.iloc[0]['uniqueIndex']

                # get complete name
                str_completeName = _get_complete_name(df_year, uniqueID)

        		# write complete name to list of last names
                df_year = split_lastname(df_year, lastname, uniqueID, str_completeName, bln_unique = True)

    	    # if there are several people with the same last name
            else:
        		# write last name and canton to correct list
                for idx, row in df_temp.drop_duplicates(['uniqueIndex']).iterrows():
        		    # extract unique index
                    uniqueID = df_temp.loc[idx]['uniqueIndex']

                    # get complete name
                    str_completeName = _get_complete_name(df_year, uniqueID)

        		    # write the lastname to the list
                    df_year = split_lastname(df_year, lastname, uniqueID, str_completeName, bln_unique = False)

        return df_year

    def extract(self):
        # read excel file and save first sheet to a dataframe
        _xl = pd.ExcelFile(self.input_file)
        _str_sheetname = _xl.sheet_names[0]
        _orddict = _xl.parse([_str_sheetname])
        _df = _orddict[_str_sheetname]

        # drop duplicate entries
        _df = _df.drop_duplicates()

        # extract all people participating after 1890
        # starting from 1891
        _df1 = _df[pd.to_datetime(_df['DateLeaving']) > datetime.datetime(1890, 12, 31)]
        # get rid of people with wrong DateLeaving that is encoded as 1899-12-30 00:00:00 instead of dd.mm.yyyy
        _df1 = _df1[_df1['DateLeaving'].str.len() == 10]
        # current members
        _df2 = _df[_df['DateLeaving'].isnull()]
        # combine these dataframes
        _df_after1890 = pd.concat([_df1, _df2])

        # generate unique ID for every person
        # generate now columns
        _df_after1890 = _df_after1890.assign(uniqueIndex=0)
        _df_after1890 = _df_after1890.assign(additionalInfo='')
        _df_after1890 = _df_after1890.assign(additionalInfo2='')
        _df_after1890 = _df_after1890.assign(additionalInfo3='')

        # group by first and last name, and date of birth
        _grouped = _df_after1890.groupby(["LastName", "FirstName", "DateOfBirth"])

        # assign first index to all entries of a person
        for list_index in _grouped.groups.values():
            _df_after1890.loc[list_index, 'uniqueIndex'] = list_index[0]

        # some people need additional information such as a place or a second last name to be uniquely identified
        df_addInfo = pd.read_csv(self.input_file_addInfo)
        for row in df_addInfo.itertuples(index=False, name='Pandas'):
            _df_after1890.loc[(_df_after1890['LastName'] == row[0]) & (_df_after1890['FirstName'] == row[1]) & (_df_after1890['CantonAbbreviation'] == row[2]), 'additionalInfo'] = row[3]
            _df_after1890.loc[(_df_after1890['LastName'] == row[0]) & (_df_after1890['FirstName'] == row[1]) & (_df_after1890['CantonAbbreviation'] == row[2]), 'additionalInfo2'] = row[4]
            _df_after1890.loc[(_df_after1890['LastName'] == row[0]) & (_df_after1890['FirstName'] == row[1]) & (_df_after1890['CantonAbbreviation'] == row[2]), 'additionalInfo3'] = row[5]

        # write dataframe to csv
        _df_after1890.to_csv(self.output_file_csv)

        # for every year
        for year in self.range_years:

            # extract every MP that was active in that year
            # (every MP of a year joined before the end of the and left after the beginning of the year)
            df_year = _df_after1890[pd.to_datetime(_df_after1890['DateJoining']) <= datetime.datetime(year, 12, 31, 0, 0)]
            df_year = df_year[pd.to_datetime(df_year['DateLeaving']) >= datetime.datetime(year, 1, 1, 0, 0)]
            print(year, df_year.shape)

            # generate new column for name type and short and complete name
            df_year = df_year.assign(nameType='')
            df_year = df_year.assign(shortName='')
            df_year = df_year.assign(completeName='')

            # refine yearly dataframe for name disambiguation
            # !!! list contains errors, see definition of function
            df_year = self.refine_yearly_dataframe(df_year)
            #print(df_year)

            # dump dataframe to a pickle file
            if not os.path.exists(self.output_folder_pickle):
                os.makedirs(self.output_folder_pickle)
            with open(self.output_folder_pickle + str(year) + "_MPs.pickle", 'wb') as f:
                pickle.dump(df_year, f)

            # write dataframe to csv
            if not os.path.exists(self.output_folder_csv):
                os.makedirs(self.output_folder_csv)
            df_year.to_csv(self.output_folder_csv + str(year) + "_MPs.csv")


# years of interest
years = [1891, 2016]   #2016

mps_extractor = MPs_Extractor(years, input_file, input_file_addInfo, output_file_csv, output_folder_pickle, output_folder_csv)
mps_extractor.extract()
