#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Code to get titles of all the documents and save it in a csv file
# to run:
# renku run --isolation python src/python/extract_document_titles.py data/AB/ 03_correctedmeta data/lists/all_titles.csv

import sys
import csv

import utils_proc
import def_classes as defc

# years of interest
years = [1891, 1995]   #1995
range_years = range(years[0], years[1] + 1)

# specify input and output parameters
folder_database = sys.argv[1]
suffix_correctedmeta = sys.argv[2]
output_titles = sys.argv[3]

# title attributes
list_attributes = ['year', 'number', 'TITEL_NORMAL_DE', 'TITEL_NORMAL_FR', 'TITEL_ORIGINAL_DE', 'TITEL_ORIGINAL_FR']

# initialize list of all titles and add header
list_all_titles = []
list_all_titles.append(list_attributes)

# for each year
for year in range_years:
    print(year)
    # generate path of input file
    input_file = folder_database + str(year) + '/' + suffix_correctedmeta + '.tar.gz'
    command = 'git lfs pull -I ' + input_file
    utils_proc.call_with_out(command)

    # get list of files to process and sort that list
    files_to_process, _ = utils_proc.get_list(str(year), folder_database, suffix_correctedmeta)
    files_to_process.sort()

    # for each file
    for file_tarpath in files_to_process:
        # get id
        id_doc = file_tarpath.split('/')[-1][:8]

        # instantiate document object (always from original pdf)
        file_aux = str(year) + '/' + id_doc + '.pdf'
        file_doc = defc.Document(file_aux, folder_database)

        # add year and id
        list_titles = [str(year), id_doc]

        # get titles and add them to the list for that document
        file_doc.get_document_title(list_attributes[2:])
        list_titles.extend(file_doc.list_titles)

        # append to list of all titles
        list_all_titles.append(list_titles)

# save csv file
with open(output_titles, 'w') as fo:
    wr = csv.writer(fo, dialect='excel')
    wr.writerows(list_all_titles)
