#!/usr/bin/env python3


from spacy import displacy
colors = {'ORG': '#73c6b6', 'LOC': '#bb8fce', 'PER': '#e59866', 'MISC': '#a6acaf'}
options = {'ents': ['ORG', 'LOC', 'PER', 'MISC'], 'colors': colors}
colors_sner = {'ORGANIZATION': '#73c6b6', 'LOCATION': '#bb8fce', 'PERSON': '#e59866', 'MISC': '#a6acaf'}
options_sner = {'ents': ['ORGANIZATION', 'LOCATION', 'PERSON', 'MISC'], 'colors': colors_sner}
colors_sner_french = {'I-ORG': '#73c6b6', 'I-LIEU': '#bb8fce', 'I-PERS': '#e59866'}
options_sner_french = {'ents': ['I-ORG', 'I-LOC', 'I-PERS'], 'colors': colors_sner_french}



def get_spacyformat_for_sner_output(text):
    if isinstance(text, tuple):
        title = text[0]
        sometext = text[1]
    elif isinstance(text, str):
        title = None
        sometext = text
    
    # generate sner output as a list
    list_tags_all = tagger.get_entities(sometext)
    
    # initiliaze dictionary
    somedict = {}
    
    # specify text and title
    somedict['text'] = sometext
    if isinstance(title, str):
        somedict['title'] = title
    
    # generate list of entities with corresponding start und end indices
    list_ents = []
    remtext = sometext
    overall_start_index = 0
    for ent in list_tags_all:
        start_index = remtext.index(ent[0])
        overall_start_index += start_index
        end_index = start_index + len(ent[0])
        overall_end_index = overall_start_index + len(ent[0])
        #print(ent, remtext[start_index:end_index], sometext[overall_start_index:overall_end_index])
        remtext = remtext[end_index+1:]
    
        if ent[1] != 'O':
            list_ents.append({'start': overall_start_index, 'end': overall_end_index, 'label': ent[1]})
            #print(ent)
        overall_start_index += len(ent[0]) + 1
    
    # add that to dictionary
    somedict['ents'] = list_ents
    
    return somedict

def write_to_txt(alldicts, filename):
    f = open(filename, 'w')
    for key, value in alldicts.items():
        f.write( str(value) )
        f.write( '\n\n.\n\n')
    f.write('generated: ' + str(datetime.datetime.now()))
    f.close()

def read_from_txt(filename):
    f = open(filename,'r')
    data=f.read()
    f.close()
    alldicts = {}
    for sent in data.split('\n\n.\n\n'):
        #print(sent)
        try:
            somedict = eval(sent)
            key = somedict['title'].split('/')[-1]
            alldicts[key] = somedict
        except SyntaxError:
            #print('SyntaxError')
            pass
    return alldicts

def render_dict(alldicts, language):
    if language == 'german':
        options = options_sner
    if language == 'french':
        options = options_sner_french
    for sent_key, sent_dict in alldicts.items():
        displacy.render(sent_dict, style='ent', jupyter=True, manual=True, options=options)


def transform_to_training_format(alldicts):
    
    def get_entitities_in_training_format(list_ents):

        list_ents_train = []
        for dict_ent in list_ents:
            start = dict_ent['start']
            end = dict_ent['end']
            label = dict_ent['label']
            if label == 'PERSON':
                label = 'PER'
            if label == 'ORGANIZATION':
                label = 'ORG'
            if label == 'LOCATION':
                label = 'LOC'
            tpl_ent = (start, end, label)

            list_ents_train.append(tpl_ent)

        return list_ents_train
    
    train_data = []

    for speaker, somedict in alldicts.items():
        text = somedict['text']
        ents_as_list = somedict['ents']
        ents_in_dict = {}
        ents_in_dict['entities'] = get_entitities_in_training_format(ents_as_list)
        tpl = (text, ents_in_dict)
        train_data.append(tpl)

    return train_data

def get_language(filepath):
    language = 'french' if filepath.endswith('_french.txt') else 'german'
    return language
