#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Sep 28 13:44:58 2018

@author: luissalamanca
"""

# Just some useful various functions

import os
import numpy as np
import git
import subprocess
import tarfile
from pdf2image import convert_from_path, convert_from_bytes
import re
import xml.etree.ElementTree as ET

#git_repo = git.Repo(os.getcwd(), search_parent_directories=True)
#git_root = git_repo.git.rev_parse("--show-toplevel")

PATH_PDF2TXT = 'pdf2txt.py'
#print(PATH_PDF2TXT)
#PATH_PDF2TXT = "/Users/luissalamanca/anaconda3/envs/py36/bin/pdf2txt.py"
HEIGHT_CHAR = 12
WIDTH_CHAR = 6


def call_with_out(full_comm):
    ## call date command ##
    #print(full_comm)
    p = subprocess.Popen(full_comm, stdout=subprocess.PIPE, shell=True) 
    (output, err) = p.communicate()
    p_status = p.wait() ## Wait for function to terminate.
    #print("Command output : ", output)
#    print("Command exit status/return code : ", p_status)
    return output
 
# This is a bit cumbersome, but wand.image is not importing the pdf from Spyder, 
# but it is working from the terminal. Thus, I just run function 
def pdf2png(input_file, res = 300, output_file = '', output_path = ''):    
    if len(output_file) == 0:
        output_file = os.path.split(os.path.splitext(input_file)[0])[1]
    if len(output_path) == 0:
        output_path = os.path.split(os.path.splitext(input_file)[0])[0]        
    convert_from_path(input_file, dpi = res, output_folder = output_path, fmt = 'png')

def pdf2xml(input_file, output_file = '', output_path = '', fmt = 'xml', suffix_str = '_data', 
            page_n = 1, flag_end = 0):
    # Also to txt or html
    if len(output_file) == 0:
        output_file = os.path.split(os.path.splitext(input_file)[0])[1]
    if len(output_path) == 0:
        output_path = os.path.split(os.path.splitext(input_file)[0])[0] 
    ext = fmt
    if fmt == 'text':
        ext = 'txt'  
        
    string_nums = np.array2string(page_n[0])
    for str in page_n[1:]:
        string_nums = string_nums + ',' + np.array2string(str)
    name_out = output_path + "/" + output_file + suffix_str + "." + ext
    
    if flag_end:
        full_comm = PATH_PDF2TXT + " -o " \
            + name_out + " -t " + fmt + " -p " \
            + string_nums + " " + input_file   
    else:
        full_comm = "python " + PATH_PDF2TXT + " " + input_file + " -o " \
                    + name_out + " -t " + fmt + " -p " \
                    + string_nums

             
#    print(full_comm)
    call_with_out(full_comm)
    return name_out

def get_list(year, folder_database, name_file):
    with tarfile.open(folder_database + '/' + str(year) +'/' + name_file + '.tar.gz') as tar:
       members = tar.getmembers()
    
       files_aux = [
           tarinfo for tarinfo in members
           if tarinfo.isfile()           
       ]
       
       files = list()
       list_ids = list()
       for f in files_aux:
           aux_l = str(f).split(' ')[1][1:-1]
           files.append(aux_l)
           list_ids.append(aux_l.split('/')[-1].split('.')[0])
           
       list_ids = np.unique(np.array(list_ids)).tolist()
       tar.close()
       return files, list_ids
   
def get_handlerfile(input_file, folder_database, name_file):
    if input_file[:2] != './':
        year = input_file.split('/')[0]  
        input_file = './' + input_file 
    else:
        year = input_file.split('/')[1]
    return tarfile.open(folder_database + '/' + str(year) +'/' + name_file + '.tar.gz').extractfile(input_file)
    
def tar_extractfile(input_file, folder_database, name_file):
    if input_file[:2] != './':
        year = input_file.split('/')[0]  
        input_file = './' + input_file 
    else:
        year = input_file.split('/')[1] 
    with tarfile.open(folder_database + '/' + str(year) +'/' + name_file + '.tar.gz') as tar:
        tar.extract(input_file)    
        tar.close()
        return input_file
    
def addto_tar(input_file, folder_database, name_file):
    if input_file[:2] != './':
        year = input_file.split('/')[0]  
        input_file = './' + input_file 
    else:
        year = input_file.split('/')[1] 
    name_tar = folder_database + '/' + str(year) +'/' + name_file + '.tar.gz'  
    if os.path.isfile(name_tar):
        list_files_intar = get_list(year, folder_database, name_file)[0]
        # Just to check if the file exists already inside the tar. In that case, we
        # first extract everything, except for it, and tar everything again
        if input_file in list_files_intar:
            for in_f in list_files_intar:
                if in_f!= input_file:
                    #print(in_f, folder_database, name_file)
                    tar_extractfile(in_f, folder_database, name_file)
            tf = tarfile.open(name_tar, mode="w")
            for in_f in list_files_intar:
                tf.add(in_f)
                    
        else:
            tf = tarfile.open(name_tar, mode="a")
            tf.add(input_file)
    else:
        tf = tarfile.open(name_tar, mode="w")
        tf.add(input_file)
    tf.close()
    return name_tar

def compress_tar(infile, outname = ''):
    if len(outname) == 0:
        outname = infile
    
    # tar -xf data/AB/${year}/02_extractedxml.tar.gz
    # tar -czvf data/AB/${year}/02_extractedxml.tar.gz ./${year}/
    # rm -rf ${year}
    year = infile.split('/')[-2] 
    c1 = 'tar -xf ' + infile
    call_with_out(c1)
    c2 = 'tar -czvf ' + outname + ' ./' + str(year) + '/'
    call_with_out(c2)
    c3 = 'rm -rf ' + str(year)
    call_with_out(c3)
    

def correct_metadata(year, id_doc, flag_end):
    
    keywords = ('In','Jahr','Band','Session','Rat','Sitzung','Geschäftsnummer',
                'Datum','Seite','Ref. No')
    max_sep = 6 # Just a parameter to capture the textlines from the db file
    
    full_path = str(year) + '/' + str(id_doc)
    command = 'cp ' + full_path + '.db ' + full_path + '_db.pdf'
    call_with_out(command)
    name_xml = pdf2xml(full_path + '_db.pdf', page_n = np.array([1]), suffix_str = '', flag_end = flag_end)
    
    XML_tree = ET.parse(name_xml)
    XML_root = XML_tree.getroot()

    coord_textline = np.array([]).reshape((6,0))
    all_text_list = list()
    for ind_el in range(len(XML_root[0])):
        for ind_line in range(len(XML_root[0][ind_el])):
            if XML_root[0][ind_el][ind_line].tag == 'textline':
                coord_textline_aux = np.array(XML_root[0][ind_el][ind_line].attrib['bbox'].split(',')).astype(np.float64)
                coord_textline_aux = np.concatenate((coord_textline_aux, np.array([ind_el,ind_line])))
                coord_textline = np.concatenate((coord_textline, np.array(coord_textline_aux).reshape((6,1))), axis = 1)
                text_str = ''
                aux_str = text_str.join([XML_root[0][ind_el][ind_line][ind_t].text for ind_t in range(len(XML_root[0][ind_el][ind_line]))])
                all_text_list.append(aux_str)
                
    xml_extrameta = ET.Element('META_FROM_DB')
    for keyw in keywords:
        #print(keyw)
        ind_textl = np.min(np.argwhere(np.array([all_text_list]) == keyw + '\n')[:,1])
        coord_r = coord_textline[1,ind_textl]
        distance = abs(coord_textline[1,:] - coord_r)
        ind_valtextl = np.setdiff1d(np.argwhere(distance < max_sep),ind_textl)[0]
        if ind_valtextl.size:
            xml_extrameta.attrib[keyw.replace('. ','').upper()] = all_text_list[ind_valtextl][:-1]
        else:
            xml_extrameta.attrib[keyw.replace('. ','').upper()] = ''
            
    path_xml_meta = full_path + '.xml'
    XML_tree_meta = ET.parse(path_xml_meta)
    XML_root_meta = XML_tree_meta.getroot()
    XML_root_meta[0].append(xml_extrameta)
    
    tree = ET.ElementTree(XML_root_meta)
    tree.write(full_path + '_metacorr.xml', encoding = 'utf-8')
    return full_path + '_metacorr.xml'
